//
//	fancyfileselection.cc
//

#include "wx/filedlg.h"
#include "wx/window.h"

#include <sstream>
#include "fancyfileselection.h"
#include "lang.h"
#include "string.h"


static struct {
	const char *name, *extension;
} file_formats[] = {
	{ "GraphThing", ".gt" },
	{ "GraphML", ".graphml" },
	{ "Graphviz", ".dot" },
	{ "Maple", ".mws" },
	{ "Postscript", ".ps" }
};

FancyFileSelection::FancyFileSelection (wxWindow *parent, const String &title,
					long style, bool file_types,
					const String &mask)
		: wxFileDialog (parent, title.c_str (), "", "", "*.*", style)
{
	if (style & wxOPEN)
		style |= wxHIDE_READONLY;
	else if (style & wxSAVE)
		style |= wxOVERWRITE_PROMPT;
	SetStyle (style);

	if (file_types) {
		wxString wc = "";
		int num = sizeof (file_formats) / sizeof (file_formats[0]);
		for (int i = 0; i < num; ++i) {
			if (i > 0)
				wc += "|";
			wc += file_formats[i].name;
			wc += " (*";
			wc += file_formats[i].extension;
			wc += ")|*";
			wc += file_formats[i].extension;
			wc += "";
		}
		SetWildcard (wc);
	} else {
		SetWildcard (mask.c_str ());
	}
}

String FancyFileSelection::GetFileType () const
{
	return String (file_formats[GetFilterIndex ()].name);
}

String FancyFileSelection::GetFileName () const
{
	return String (wxFileDialog::GetPath ().c_str ());
}
