//
//	gui.cc
//

#include "wx/bitmap.h"
#include "wx/frame.h"
#include "wx/gauge.h"
#include "wx/icon.h"
#include "wx/menu.h"
#include "wx/msgdlg.h"
#include "wx/statusbr.h"
#include "wx/toolbar.h"

#include <math.h>
#include "canvas.h"
#include "config.h"
#include "edge.h"
#include "factory.h"
#include "graph.h"
#include "gui.h"
#include "lang.h"
#include "string.h"
#include "undo.h"
#include "vertex.h"


// Toolbar pixmaps
#include "edge_mode.xpm"
#include "vertex_mode.xpm"

#include  "logo.xpm"


BEGIN_EVENT_TABLE(GTFrame, wxFrame)
	EVT_SIZE	(GTFrame::OnSize)

	EVT_MENU	(ID_FILE_LOAD,		GTFrame::cb_File_Load)
	EVT_MENU	(ID_FILE_SAVE,		GTFrame::cb_File_Save)
	EVT_MENU	(ID_FILE_EXIT,		GTFrame::cb_File_Exit)

	EVT_MENU	(ID_EDIT_UNDO,		GTFrame::cb_Edit_Undo)
	EVT_MENU	(ID_EDIT_SELECTALL,	GTFrame::cb_Edit_SelectAll)
	EVT_MENU	(ID_EDIT_SELECTNONE,	GTFrame::cb_Edit_SelectNone)
	EVT_MENU	(ID_EDIT_INVERTSELV,	GTFrame::cb_Edit_InvertSelectionVertices)
	EVT_MENU	(ID_EDIT_INVERTSELE,	GTFrame::cb_Edit_InvertSelectionEdges)
	EVT_MENU	(ID_EDIT_INVERTSELA,	GTFrame::cb_Edit_InvertSelectionAll)

	EVT_MENU	(ID_VIEW_LABELS,	GTFrame::cb_View)
	EVT_MENU	(ID_VIEW_WEIGHTS,	GTFrame::cb_View)

	EVT_MENU	(ID_GRAPH_CLEAR,	GTFrame::cb_Graph_Clear)
	EVT_MENU	(ID_GRAPH_COMPLEMENT,	GTFrame::cb_Graph_Complement)
	EVT_MENU	(ID_GRAPH_LINEGRAPH,	GTFrame::cb_Graph_LineGraph)
	EVT_MENU	(ID_GRAPH_SUBGRAPH,	GTFrame::cb_Graph_Subgraph)

	EVT_MENU	(ID_GRAPH_FIND_SHORTESTPATH,	GTFrame::cb_Graph_Find_ShortestPath)
	EVT_MENU	(ID_GRAPH_FIND_BFS,	GTFrame::cb_Graph_Find_BFS)
	EVT_MENU	(ID_GRAPH_FIND_DFS,	GTFrame::cb_Graph_Find_DFS)
	EVT_MENU	(ID_GRAPH_FIND_MST,	GTFrame::cb_Graph_Find_MST)

	EVT_MENU	(ID_GRAPH_PROPERTIES_CONNECTIVITY,	GTFrame::cb_Graph_Properties_Connectivity)
	EVT_MENU	(ID_GRAPH_PROPERTIES_EULERICITY,	GTFrame::cb_Graph_Properties_Eulericity)
	EVT_MENU	(ID_GRAPH_PROPERTIES_HAMILTONICITY,	GTFrame::OnMenu)
	EVT_MENU	(ID_GRAPH_PROPERTIES_PLANARITY,		GTFrame::OnMenu)

	EVT_MENU	(ID_GRAPH_STATISTICS_ADJMATRIX,	GTFrame::cb_Graph_Statistics_AdjacencyMatrix)
	EVT_MENU	(ID_GRAPH_STATISTICS_DEGSEQ,	GTFrame::cb_Graph_Statistics_DegreeSequence)
	EVT_MENU	(ID_GRAPH_STATISTICS_DIAMETER,	GTFrame::cb_Graph_Statistics_Diameter)
	EVT_MENU	(ID_GRAPH_STATISTICS_GIRTH,	GTFrame::OnMenu)
	EVT_MENU	(ID_GRAPH_STATISTICS_RADIUS,	GTFrame::cb_Graph_Statistics_Radius)
	EVT_MENU	(ID_GRAPH_STATISTICS_CHROMNUM,	GTFrame::cb_Graph_Statistics_ChromaticNumber)
	EVT_MENU	(ID_GRAPH_STATISTICS_CHROMINDEX,	GTFrame::cb_Graph_Statistics_ChromaticIndex)
	EVT_MENU	(ID_GRAPH_STATISTICS_CHROMPOLY,	GTFrame::cb_Graph_Statistics_ChromaticPolynomial)

	EVT_MENU	(ID_PREFAB_COMPLETE,	GTFrame::cb_Prefab_Complete)
	EVT_MENU	(ID_PREFAB_COMPLETEBIPARTITE,	GTFrame::cb_Prefab_CompleteBipartite)
	EVT_MENU	(ID_PREFAB_CYCLE,	GTFrame::cb_Prefab_Cycle)
	EVT_MENU	(ID_PREFAB_GEAR,	GTFrame::cb_Prefab_Gear)
	EVT_MENU	(ID_PREFAB_HANOI,	GTFrame::cb_Prefab_Hanoi)
	EVT_MENU	(ID_PREFAB_LADDER,	GTFrame::cb_Prefab_Ladder)
	EVT_MENU	(ID_PREFAB_LATTICE,	GTFrame::cb_Prefab_Lattice)
	EVT_MENU	(ID_PREFAB_NULL,	GTFrame::cb_Prefab_Null)
	EVT_MENU	(ID_PREFAB_STAR,	GTFrame::cb_Prefab_Star)
	EVT_MENU	(ID_PREFAB_TREE,	GTFrame::OnMenu)
	EVT_MENU	(ID_PREFAB_WHEEL,	GTFrame::cb_Prefab_Wheel)
	EVT_MENU	(ID_PREFAB_PETERSEN,	GTFrame::cb_Prefab_Petersen)

	EVT_MENU	(ID_PREFAB_PLATONIC_TETRAHEDRAL,	GTFrame::cb_Prefab_Platonic_Tetrahedral)
	EVT_MENU	(ID_PREFAB_PLATONIC_CUBICAL,		GTFrame::cb_Prefab_Platonic_Cubical)
	EVT_MENU	(ID_PREFAB_PLATONIC_OCTAHEDRAL,		GTFrame::cb_Prefab_Platonic_Octahedral)
	EVT_MENU	(ID_PREFAB_PLATONIC_DODECAHEDRAL,	GTFrame::cb_Prefab_Platonic_Dodecahedral)
	EVT_MENU	(ID_PREFAB_PLATONIC_ICOSAHEDRAL,	GTFrame::cb_Prefab_Platonic_Icosahedral)

	EVT_MENU	(ID_HELP_ABOUT,		GTFrame::cb_Help_About)

	EVT_TOOL	(ID_TOOL_VERTEXMODE,	GTFrame::cb_Change_Mode)
	EVT_TOOL	(ID_TOOL_EDGEMODE,	GTFrame::cb_Change_Mode)
END_EVENT_TABLE()

wxMenuBar *GTFrame::genMenuBar ()
{
	// File Menu
	wxMenu *menu_file = new wxMenu ();
	menu_file->Append (ID_FILE_LOAD, _wx("&Load\tCtrl-O"));
	menu_file->Append (ID_FILE_SAVE, _wx("&Save\tCtrl-S"));
	menu_file->AppendSeparator ();
	menu_file->Append (ID_FILE_EXIT, _wx("E&xit\tCtrl-Q"));

	// Edit Menu
	wxMenu *menu_edit = new wxMenu ();
	menu_edit->Append (ID_EDIT_UNDO, _wx("&Undo\tCtrl-Z"));
	menu_edit->FindItem (ID_EDIT_UNDO)->Enable (false);
	menu_edit->AppendSeparator ();
	menu_edit->Append (ID_EDIT_SELECTALL, _wx("Select &All\tCtrl-A"));
	menu_edit->Append (ID_EDIT_SELECTNONE, _wx("Select &None"));
	menu_edit->Append (ID_EDIT_INVERTSELV, _wx("Invert Selection (&Vertices)"));
	menu_edit->Append (ID_EDIT_INVERTSELE, _wx("Invert Selection (&Edges)"));
	menu_edit->Append (ID_EDIT_INVERTSELA, _wx("&Invert Selection (All)"));

	// View Menu
	wxMenu *menu_view = new wxMenu ();
	menu_view->AppendCheckItem (ID_VIEW_LABELS, _wx("&Labels"));
	menu_view->AppendCheckItem (ID_VIEW_WEIGHTS, _wx("&Weights"));

	// Graph Menu
	wxMenu *menu_graph = new wxMenu ();
	menu_graph->Append (ID_GRAPH_CLEAR, _wx("&Clear\tCtrl-W"));
	menu_graph->Append (ID_GRAPH_COMPLEMENT, _wx("C&omplement"));
	menu_graph->Append (ID_GRAPH_LINEGRAPH, _wx("&Line Graph"));
	menu_graph->Append (ID_GRAPH_SUBGRAPH, _wx("S&ubgraph"));
	menu_graph->AppendSeparator ();

	// Graph/Find submenu
	wxMenu *menu_graph_find = new wxMenu ();
	menu_graph_find->Append (ID_GRAPH_FIND_SHORTESTPATH,
					_wx("&Shortest Path"));
	menu_graph_find->Append (ID_GRAPH_FIND_BFS,
					_wx("&Breadth-First Search"));
	menu_graph_find->Append (ID_GRAPH_FIND_DFS,
					_wx("&Depth-First Search"));
	menu_graph_find->Append (ID_GRAPH_FIND_MST,
					_wx("&Minimum Spanning Tree"));
	menu_graph->Append (ID_GRAPH_FIND, _wx("&Find"), menu_graph_find);

	// Graph/Properties submenu
	wxMenu *menu_graph_prop = new wxMenu ();
	menu_graph_prop->Append (ID_GRAPH_PROPERTIES_CONNECTIVITY,
					_wx("&Connectivity"));
	menu_graph_prop->Append (ID_GRAPH_PROPERTIES_EULERICITY,
					_wx("&Eulericity"));
	menu_graph_prop->Append (ID_GRAPH_PROPERTIES_HAMILTONICITY,
					_wx("&Hamiltonicity"));
	menu_graph_prop->FindItem (ID_GRAPH_PROPERTIES_HAMILTONICITY)->Enable (false);
	menu_graph_prop->Append (ID_GRAPH_PROPERTIES_PLANARITY,
					_wx("&Planarity"));
	menu_graph_prop->FindItem (ID_GRAPH_PROPERTIES_PLANARITY)->Enable (false);
	menu_graph->Append (ID_GRAPH_PROPERTIES, _wx("&Properties"), menu_graph_prop);

	// Graph/Statistics submenu
	wxMenu *menu_graph_stat = new wxMenu ();
	menu_graph_stat->Append (ID_GRAPH_STATISTICS_ADJMATRIX,
					_wx("&Adjacency Matrix"));
	menu_graph_stat->Append (ID_GRAPH_STATISTICS_DEGSEQ,
					_wx("&Degree Sequence"));
	menu_graph_stat->Append (ID_GRAPH_STATISTICS_DIAMETER,
					_wx("D&iameter"));
	menu_graph_stat->Append (ID_GRAPH_STATISTICS_GIRTH,
					_wx("&Girth"));
	menu_graph_stat->FindItem (ID_GRAPH_STATISTICS_GIRTH)->Enable (false);
	menu_graph_stat->Append (ID_GRAPH_STATISTICS_RADIUS,
					_wx("&Radius"));
	menu_graph_stat->AppendSeparator ();
	menu_graph_stat->Append (ID_GRAPH_STATISTICS_CHROMNUM,
					_wx("&Chromatic Number"));
	menu_graph_stat->Append (ID_GRAPH_STATISTICS_CHROMINDEX,
					_wx("C&hromatic Index"));
	menu_graph_stat->Append (ID_GRAPH_STATISTICS_CHROMPOLY,
					_wx("Chromatic &Polynomial"));
	menu_graph->Append (ID_GRAPH_STATISTICS, _wx("&Statistics"), menu_graph_stat);

	// Prefab Menu
	wxMenu *menu_prefab = new wxMenu ();
	menu_prefab->Append (ID_PREFAB_COMPLETE, _wx("Complete (&Kn)"));
	menu_prefab->Append (ID_PREFAB_COMPLETEBIPARTITE,
					_wx("Complete &Bipartite (Kn,m)"));
	menu_prefab->Append (ID_PREFAB_CYCLE, _wx("Cycle (&Cn)"));
	menu_prefab->Append (ID_PREFAB_GEAR, _wx("Gear (&Gn)"));
	menu_prefab->Append (ID_PREFAB_HANOI, _wx("Hanoi (&Hn)"));
	menu_prefab->Append (ID_PREFAB_LADDER, _wx("Ladder (&Ln)"));
	menu_prefab->Append (ID_PREFAB_LATTICE, _wx("Lattice (Ln,m)"));
	menu_prefab->Append (ID_PREFAB_NULL, _wx("Null (&Nn)"));
	menu_prefab->Append (ID_PREFAB_STAR, _wx("Star (&Sn)"));
	menu_prefab->Append (ID_PREFAB_TREE, _wx("Tree (&Tn)"));
	menu_prefab->FindItem (ID_PREFAB_TREE)->Enable (false);
	menu_prefab->Append (ID_PREFAB_WHEEL, _wx("Wheel (&Wn)"));
	menu_prefab->AppendSeparator ();
	menu_prefab->Append (ID_PREFAB_PETERSEN, _wx("Petersen"));

	// Prefab/Platonic submenu
	wxMenu *menu_prefab_platonic = new wxMenu ();
	menu_prefab_platonic->Append (ID_PREFAB_PLATONIC_TETRAHEDRAL,
							_wx("&Tetrahedral"));
	menu_prefab_platonic->Append (ID_PREFAB_PLATONIC_CUBICAL,
							_wx("&Cubical"));
	menu_prefab_platonic->Append (ID_PREFAB_PLATONIC_OCTAHEDRAL,
							_wx("&Octahedral"));
	menu_prefab_platonic->Append (ID_PREFAB_PLATONIC_DODECAHEDRAL,
						       	_wx("&Dodecahedral"));
	menu_prefab_platonic->Append (ID_PREFAB_PLATONIC_ICOSAHEDRAL,
							_wx("&Icosahedral"));
	menu_prefab->Append (ID_PREFAB_PLATONIC, _wx("&Platonic"), menu_prefab_platonic);

	// Help Menu
	wxMenu *menu_help = new wxMenu ();
	menu_help->Append (ID_HELP_ABOUT, _wx("&About\tF1"));

	// Menubar
	wxMenuBar *mb = new wxMenuBar ();
	mb->Append (menu_file, _wx("&File"));
	mb->Append (menu_edit, _wx("&Edit"));
	mb->Append (menu_view, _wx("&View"));
	mb->Append (menu_graph, _wx("&Graph"));
	mb->Append (menu_prefab, _wx("&Prefab"));
	mb->Append (menu_help, _wx("&Help"));
	// TODO: right-align help menu

	return mb;
}

void GTFrame::OnMenu (wxCommandEvent &event)
{
	// TODO
	wxMessageDialog dlg (this, "This isn't implemented yet!", "NYI.",
						wxOK | wxICON_ERROR);
	dlg.ShowModal ();
}

#include "wx/settings.h"
void GTFrame::OnSize (wxSizeEvent &event)
{
	wxRect rect;

	// Fix up progress bar
	if (!statusBar)
		return;
	statusBar->GetFieldRect (2, rect);

	progressBar->SetSize (rect.x + 2, rect.y + 2, rect.width - 4, rect.height - 4);
	wxSize sz = progressBar->GetSize ();
	progressBar->Move (rect.x + (rect.width - sz.x) / 2,
				rect.y + (rect.height - sz.y) / 2);

	// Fix up canvas
	sz = GetClientSize ();
	int nudge_horiz = wxSystemSettings::GetMetric (wxSYS_VSCROLL_X);
	int nudge_vert = wxSystemSettings::GetMetric (wxSYS_HSCROLL_Y);
	sz.SetWidth (sz.GetWidth () - nudge_horiz);
	sz.SetHeight (sz.GetHeight () - nudge_vert);
	canvas->SetClientSize (sz.GetWidth (), sz.GetHeight ());
	Factory::width = sz.GetWidth ();
	Factory::height = sz.GetHeight ();
}

void GTFrame::setUndoText (String description)
{
	String s_txt (_("&Undo"));
	if (description != "")
		s_txt += " (" + description + ")";
	s_txt += "\tCtrl-Z";
	edit_Undo->SetText (s_txt.c_str ());
}

GTFrame::GTFrame ()
	: wxFrame ((wxFrame *) NULL, -1, "GraphThing " GT_VERSION,
				wxDefaultPosition, wxSize (600, 500))
{
	SetIcon (wxIcon (logo_xpm));

	graph = new Graph ();


	SetMenuBar (genMenuBar ());

	edit_Undo = GetMenuBar ()->FindItem (ID_EDIT_UNDO);
	view_Labels = GetMenuBar ()->FindItem (ID_VIEW_LABELS);
	view_Weights = GetMenuBar ()->FindItem (ID_VIEW_WEIGHTS);

	view_Labels->Check ();


	wxToolBar *tb = CreateToolBar (wxTB_HORIZONTAL | wxTB_TEXT);

	statusBar = CreateStatusBar (3);
	int widths[] = { 0, -1, -1 };
	statusBar->SetStatusWidths (3, widths);

	progressBar = new wxGauge (statusBar, -1, 100);
	progressBar->SetValue (0);

	wxBitmap *vPix = new wxBitmap (vertex_mode_xpm);
	wxBitmap *ePix = new wxBitmap (edge_mode_xpm);
	tb->AddRadioTool (ID_TOOL_VERTEXMODE, _wx("Vertex Mode"), *vPix, *vPix,
					_wx("Change to Vertex Mode"));
	tb->AddRadioTool (ID_TOOL_EDGEMODE, _wx("Edge Mode"), *ePix, *ePix,
					_wx("Change to Edge Mode"));
	tb->Realize ();

	canvas = new Canvas (this, &graph);

	pushStatus (_("Ready."));
	wxCommandEvent ev (0, ID_TOOL_VERTEXMODE);
	cb_Change_Mode (ev);
	cb_View (ev);
}

GTFrame::~GTFrame ()
{
	delete graph;
}

void GTFrame::undoableAction (String description)
{
	UndoStep step (new Graph (*graph), description);
	undoStack.push (step);
	edit_Undo->Enable ();

	setUndoText (description);
}

void GTFrame::loadGraph (const char *fname)
{
	bool res;

	Graph *g = graph->load (fname, res);
	if (g) {
		// TODO: Make this undoable?
		delete graph;
		graph = g;
	}

	canvas->redraw ();
}

void GTFrame::msg (const String &title, const wxString &s)
{
	// TODO: pass through more flags
	wxMessageDialog dlg (this, s, title.c_str (), wxOK);
	dlg.ShowModal ();
}

void GTFrame::msg (const String &title, const String &s)
{
	// TODO: pass through more flags
	wxMessageDialog dlg (this, s.c_str (), title.c_str (), wxOK);
	dlg.ShowModal ();
}

void GTFrame::msg (const String &title, const char *s)
{
	msg (title, wxString (s));
}

void GTFrame::toggleMode ()
{
	// TODO: toggle the toolbar buttons!
}

void GTFrame::pushStatus (const String &str)
{
	statusBar->PushStatusText (wxString (str.c_str ()), 1);
}

void GTFrame::popStatus ()
{
	statusBar->PopStatusText (1);
}

void GTFrame::setProgress (double frac)
{
	static double last_frac = -1.0;

	// TODO: implement progress percentage text somehow
	if (frac < 0.0) {
		// hide progress bar
		//progressBar->set_show_text (false);
		progressBar->Hide ();
		last_frac = frac - 1;
	} else if (frac <= 1.0) {
		progressBar->SetValue (int (frac * 100));
		//progressBar->set_format_string ("%p%% complete");
		//progressBar->set_show_text (true);
		progressBar->Show ();
	}

	//progressBar->Refresh ();
	//progressBar->Update ();
}
