//
//	main.cc
//

#include "wx/app.h"

#include <iostream>
#include <stdexcept>
#include "config.h"
#include "gui.h"
#include "lang.h"


#ifdef ENABLE_NLS
Translator *translator;
#endif

class GTApp : public wxApp
{
private:
	GTFrame *win;

public:
	virtual bool OnInit ();
	virtual int OnExit ();

	void setProgress (double frac);
};

IMPLEMENT_APP(GTApp)

bool GTApp::OnInit ()
{
#ifdef ENABLE_NLS
	translator = new Translator (1);	// be noisy
	try {
		translator->init ();
	} catch (std::runtime_error e) {
		std::cerr << "Translation init failed: " << e.what () << "\n";
		return false;
	}
	translator->guess_language ();

#ifdef GT_WITH_DEV
	translator->verify_phrases ();
#endif
#endif

	win = new GTFrame ();
	win->Show (true);
	SetTopWindow (win);

	if (argc > 1)
		win->loadGraph (argv[1]);

	return true;
}

int GTApp::OnExit ()
{
	delete translator;

	return 0;
}

void GTApp::setProgress (double frac)
{
	win->setProgress (frac);
}

void setProgress (double frac)
{
	wxGetApp ().setProgress (frac);
}
